import sys
import logging
import tornado.web
import tornado.ioloop
from .handlers import ProxyHandler, ProxyWSHandler


def main(proxy_url):
    app = tornado.web.Application()
    handlers = [
        (r"/(.*)", ProxyHandler, {'proxy_url': proxy_url}),
        (r"/(.*)", ProxyWSHandler, {'proxy_url': proxy_url})
    ]
    app.add_handlers('.*$', handlers)
    logging.critical('Dashboard listening on port %s.' % 8080)
    app.listen(8080)
    try:
        tornado.ioloop.IOLoop.current().start()
    except KeyboardInterrupt:
        logging.critical('Shutting down...')


if __name__ == '__main__':
    main(sys.argv[1])



import os
import os.path
import tornado
import tornado.gen
import tornado.web
import tornado.websocket
import tornado.httpclient

ap = os.path.abspath
join = os.path.join


class ProxyHandler(tornado.web.RequestHandler):
    def initialize(self, proxy_url='/', **kwargs):
        super(ProxyHandler, self).initialize(**kwargs)
        self.proxy_url = proxy_url

    @tornado.gen.coroutine
    def get(self, url=None):
        '''Get the login page'''
        url = url or self.proxy_url
        if url is None:
            if self.request.uri.startswith('/'):
                url = self.request.uri[1:]
            else:
                url = self.request.uri

        req = tornado.httpclient.HTTPRequest(url)
        client = tornado.httpclient.AsyncHTTPClient()
        response = yield client.fetch(req, raise_error=False)

        # websocket upgrade
        if response.code == 599:
            self.set_status(200)  # switching protocols
            return

        self.set_status(response.code)
        if response.code != 200:
            self.finish()
        else:
            if response.body:
                for header in response.headers:
                    if header.lower() == 'content-length':
                        self.set_header(header, str(max(len(response.body), int(response.headers.get(header)))))
                    else:
                        if header.lower() != 'transfer-encoding':
                            self.set_header(header, response.headers.get(header))

            self.write(response.body)
            self.finish()


class ProxyWSHandler(tornado.websocket.WebSocketHandler):
    def initialize(self, proxy_url='/', **kwargs):
        super(ProxyWSHandler, self).initialize(**kwargs)
        self.proxy_url = proxy_url
        self.ws = None
        self.closed = True

    @tornado.gen.coroutine
    def open(self, url=None):
        self.closed = False
        url = url or self.proxy_url
        if url is None:
            if self.request.uri.startswith('/'):
                url = self.request.uri[1:]
            else:
                url = self.request.uri

        def write(msg):
            if self.closed:
                if self.ws:
                    self.ws.close()
                    self.ws = None
            else:
                if self.ws:
                    self.write_message(msg, binary=isinstance(msg, bytes))
        self.ws = yield tornado.websocket.websocket_connect(url,
                                                            on_message_callback=write)

    def on_message(self, message):
        if self.ws:
            self.ws.write_message(message)

    def on_close(self):
        if self.ws:
            self.ws.close()
            self.ws = None
            self.closed = True



from setuptools import setup, find_packages
from codecs import open
import io
import os
import os.path

pjoin = os.path.join
here = os.path.abspath(os.path.dirname(__file__))
name = 'tornado_proxy_handlers'


def get_version(file, name='__version__'):
    path = os.path.realpath(file)
    version_ns = {}
    with io.open(path, encoding="utf8") as f:
        exec(f.read(), {}, version_ns)
    return version_ns[name]

version = get_version(pjoin(here, name, '_version.py'))

with open(pjoin(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open(pjoin(here, 'requirements.txt'), encoding='utf-8') as f:
    requires = f.read().split()

setup(
    name=name,
    version=version,
    description='Tornado proxy handlers',
    long_description=long_description,
    url='https://github.com/timkpaine/tornado-proxy-handlers',
    author='Tim Paine',
    author_email='t.paine154@gmail.com',
    license='Apache 2.0',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    keywords='tornado proxy',
    packages=find_packages(exclude=[]),
    install_requires=requires,
    extras_require={'dev': requires + ['pytest', 'pytest-cov', 'pylint', 'flake8', 'codecov', 'bumpversion', 'mock']}
)
